﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Net;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Documents;
using System.Windows.Input;
using System.Windows.Media;
using System.Windows.Media.Animation;
using System.Windows.Shapes;
using System.Windows.Interop;
using System.Windows.Printing;
using System.ComponentModel;

namespace OutOfBrowser
{
    public partial class MainPage : UserControl, INotifyPropertyChanged
    {
        private SalesReport _salesPeople = new SalesReport();
        public SalesReport SalesPeople
        {
            get
            {
                return _salesPeople;
            }
            set
            {
                _salesPeople = value;
                if (null != PropertyChanged)
                    PropertyChanged(this, new PropertyChangedEventArgs("SalesPeople"));
            }

        }

        public MainPage()
        {
            InitializeComponent();

            InitializeSalesData(SalesPeople);
            this.DataContext = this;

            LinearGradientBrush brush = GetLinearBrush(Color.FromArgb(255, 120, 183, 225),Color.FromArgb(255, 0, 0, 200));
            LayoutRoot.Background = brush;
        }

        private void InitializeSalesData(SalesReport _salesData)
        {
            _salesData.Add(new SalesPerson("Aaberg Jesper", 123));
            _salesData.Add(new SalesPerson("Adams Ellen", 1230));
            _salesData.Add(new SalesPerson("Adams Terry", 400));
            _salesData.Add(new SalesPerson("Penor Lori", 1023));
            _salesData.Add(new SalesPerson("Pfeiffer Michael", 850));
            _salesData.Add(new SalesPerson("Philips Carol", 1250));
            _salesData.Add(new SalesPerson("Poe Toni", 370));
            _salesData.Add(new SalesPerson("Hicks  Cassie", 1340));
            _salesData.Add(new SalesPerson("Barber David", 125.8));
            _salesData.Add(new SalesPerson("Doe Jhon", 899.99));
        }

        private void btnSalesReport_Click(object sender, RoutedEventArgs e)
        {

        }       

        private void btnDeleteRow_Click(object sender, RoutedEventArgs e)
        {
            if (dgReportData.SelectedItem != null && dgReportData.SelectedItem is SalesPerson)
                SalesPeople.Remove((SalesPerson)dgReportData.SelectedItem);
            else
                MessageBox.Show("Please select a row before pressing the delete button");
        }

        private void btnAddRow_Click(object sender, RoutedEventArgs e)
        {
            // Create a new sales person
            SalesPerson newSalesPerson = new SalesPerson("New sales person", 0);
            SalesPeople.Add(newSalesPerson);

            // Set the focus to the newly created row and open it for editing
            dgReportData.SelectedItem = newSalesPerson;
            dgReportData.Focus();
            dgReportData.BeginEdit();
        }

        private void btnClearTable_Click(object sender, RoutedEventArgs e)
        {
            SalesPeople.Clear();
        }

        private static LinearGradientBrush GetLinearBrush(Color startColor, Color endColor)
        {
            LinearGradientBrush brush = new LinearGradientBrush();
            GradientStop colorStop1 = new GradientStop();
            colorStop1.Color = startColor;
            GradientStop colorStop2 = new GradientStop();
            colorStop2.Color = endColor;
            colorStop2.Offset = 1;
            brush.StartPoint = new System.Windows.Point(0.5, 1);
            brush.EndPoint = new System.Windows.Point(0.5, 0);
            brush.GradientStops = new GradientStopCollection { colorStop1, colorStop2 };
            return brush;
        }

        #region INotifyPropertyChanged Members

        public event PropertyChangedEventHandler PropertyChanged;

        #endregion      
    }
}
