﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.IO;
using System.Windows.Media.Imaging;
using System.Collections.ObjectModel;
using System.Collections;
using System.Windows.Browser;

namespace Silverlight4.Labs.PictureBrowser
{
    public partial class MainPage : UserControl
    {
        ObservableCollection<ImageInfo> images = new ObservableCollection<ImageInfo>();
        public MainPage()
        {
            InitializeComponent();
            lstPictures.DataContext = images;
            loadImagesFromServer();
            LinearGradientBrush brush = GetLinearBrush(
                Color.FromArgb(255, 23, 23, 23),
                Color.FromArgb(255, 10, 10, 10));
            LayoutRoot.Background = brush;
        }

        private void loadImagesFromServer()
        {
            BrowserHelper.BrowserHelperSoapClient client =
                    new BrowserHelper.BrowserHelperSoapClient();
            client.GetServerFileListCompleted += (s, e) =>
            {
                string[] files = e.Result.ToArray<string>();

                foreach (var file in files)
                {
                    ImageInfo imageInfo = new ImageInfo()
                    {
                        ImageName = file,
                        IsLocal = false
                    };

                    imageInfo.Image = new BitmapImage(
                    new Uri(HtmlPage.Document.DocumentUri.Scheme +
                           "://" + HtmlPage.Document.DocumentUri.Host + ":"
                           + HtmlPage.Document.DocumentUri.Port + "/Image.ashx?name=" + file));

                    imageInfo.Image.ImageOpened += (obj, args) =>
                    {
                        imageInfo.ImageHeight = (obj as BitmapImage).PixelHeight;
                        imageInfo.ImageWidth = (obj as BitmapImage).PixelWidth;
                    };

                    images.Add(imageInfo);
                }
            };

            client.GetServerFileListAsync();
        }

        private static LinearGradientBrush GetLinearBrush(Color startColor,
                                        Color endColor)
        {
            LinearGradientBrush brush = new LinearGradientBrush();
            GradientStop colorStop1 = new GradientStop();
            colorStop1.Color = startColor;
            GradientStop colorStop2 = new GradientStop();
            colorStop2.Color = endColor;
            colorStop2.Offset = 1;
            brush.StartPoint = new System.Windows.Point(0.5, 1);
            brush.EndPoint = new System.Windows.Point(0.5, 0);
            brush.GradientStops = new GradientStopCollection { colorStop1, 
								     colorStop2 };
            return brush;
        }
    }
}
