﻿using System;
using System.Linq;
using System.Windows;
using System.Windows.Controls;
using System.Windows.Input;
using System.Windows.Media;
using System.IO;
using System.Windows.Media.Imaging;
using System.Collections.ObjectModel;
using System.Collections;
using System.Windows.Browser;

namespace Silverlight4.Labs.PictureBrowser
{
    public partial class MainPage : UserControl
    {
        ObservableCollection<ImageInfo> images = new ObservableCollection<ImageInfo>();
        public MainPage()
        {
            InitializeComponent();
            lstPictures.DataContext = images;
            loadImagesFromServer();
            LinearGradientBrush brush = GetLinearBrush(
                Color.FromArgb(255, 23, 23, 23),
                Color.FromArgb(255, 10, 10, 10));
            LayoutRoot.Background = brush;
        }

        private void loadImagesFromServer()
        {
            BrowserHelper.BrowserHelperSoapClient client =
                    new BrowserHelper.BrowserHelperSoapClient();
            client.GetServerFileListCompleted += (s, e) =>
            {
                string[] files = e.Result.ToArray<string>();

                foreach (var file in files)
                {
                    ImageInfo imageInfo = new ImageInfo()
                    {
                        ImageName = file,
                        IsLocal = false
                    };

                    imageInfo.Image = new BitmapImage(
                    new Uri(HtmlPage.Document.DocumentUri.Scheme +
                           "://" + HtmlPage.Document.DocumentUri.Host + ":"
                           + HtmlPage.Document.DocumentUri.Port + "/Image.ashx?name=" + file));

                    imageInfo.Image.ImageOpened += (obj, args) =>
                    {
                        imageInfo.ImageHeight = (obj as BitmapImage).PixelHeight;
                        imageInfo.ImageWidth = (obj as BitmapImage).PixelWidth;
                    };

                    images.Add(imageInfo);
                }
            };

            client.GetServerFileListAsync();
        }

        private static LinearGradientBrush GetLinearBrush(Color startColor,
                                        Color endColor)
        {
            LinearGradientBrush brush = new LinearGradientBrush();
            GradientStop colorStop1 = new GradientStop();
            colorStop1.Color = startColor;
            GradientStop colorStop2 = new GradientStop();
            colorStop2.Color = endColor;
            colorStop2.Offset = 1;
            brush.StartPoint = new System.Windows.Point(0.5, 1);
            brush.EndPoint = new System.Windows.Point(0.5, 0);
            brush.GradientStops = new GradientStopCollection { colorStop1, 
								     colorStop2 };
            return brush;
        }

        private void OnMouseWheel(object sender, MouseWheelEventArgs e)
        {
            if (e.Delta > 0)
                slider_X.Value += slider_X.LargeChange;
            else
                slider_X.Value -= slider_X.LargeChange;
        }

        private void lstPictures_Drop(object sender, DragEventArgs e)
        {
            IDataObject @do = e.Data;
            if (@do.GetDataPresent("FileDrop"))
            {
                FileInfo[] files = @do.GetData("FileDrop") as FileInfo[];
                foreach (var file in files)
                {
                    if (file.Name.ToLower().Contains(".jpg") ||
                        file.Name.ToLower().Contains(".png"))
                    {
                        FileStream reader = file.OpenRead();

                        byte[] array = new byte[reader.Length];
                        int count = reader.Read(array, 0, (int)reader.Length);
                        MemoryStream str = new MemoryStream(array);
                        BitmapImage img = new BitmapImage();
                        img.SetSource(str);

                        ImageInfo image = new ImageInfo()
                        {
                            Image = img,
                            ImageName = file.Name,
                            ImageHeight = img.PixelHeight,
                            ImageWidth = img.PixelWidth,
                            UnderlyingStream = str,
                            IsLocal = true
                        };

                        images.Add(image);
                    }
                }

            }
        }

        private void lstPictures_MouseRightButtonDown(object sender, MouseButtonEventArgs e)
        {
            e.Handled = true;

            if ((sender as ListBox).SelectedItems.Count == 0)
                return;

            Point pos = e.GetPosition(null);
            contextMenu.SetValue(Canvas.LeftProperty, pos.X);
            contextMenu.SetValue(Canvas.TopProperty, pos.Y);
            contextMenu.Visibility = Visibility.Visible;
        }

        private void lstPictures_SelectionChanged(object sender, SelectionChangedEventArgs e)
        {
            contextMenu.Visibility = Visibility.Collapsed;
        }

        private void UserControl_MouseLeftButtonDown(object sender, MouseButtonEventArgs e)
        {
            contextMenu.Visibility = Visibility.Collapsed;
        }

        private void DeleteMenuItem_Click(object sender, RoutedEventArgs e)
        {
            IList selection = lstPictures.SelectedItems;

            foreach (var item in selection)
            {
                images.Remove(item as ImageInfo);
            }

            contextMenu.Visibility = Visibility.Collapsed;
        }
        private void UploadMenuItem_Click(object sender, RoutedEventArgs e)
        {
            foreach (var info in lstPictures.SelectedItems)
            {
                ImageInfo imageInfo = info as ImageInfo;

                if (imageInfo.IsLocal)
                {
                    BrowserHelper.BrowserHelperSoapClient client =
                    new BrowserHelper.BrowserHelperSoapClient();
                    client.UploadFileCompleted += (s, args) =>
                    {
                        if (args.Result)
                        {
                            imageInfo.IsLocal = false;
                        }
                    };

                    client.UploadFileAsync(
                    imageInfo.ImageName,
                    imageInfo.UnderlyingStream.ToArray());
                }
            }

            contextMenu.Visibility = Visibility.Collapsed;
        }
    }
}
